/* Programme de transfert de fichier/partition minimal
    compiler par diet gcc
   Franois Boisson, juin 2003 */

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <string.h>
#include <stdio.h>
#include <signal.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <unistd.h>
#include <malloc.h>

#define PORT 2003
#define TAILLE (1024*1024)

struct sockaddr_in desc_douille;
int verbeux = 0;

void erreur(message)
char *message;
{
  if (verbeux) {
    printf("Erreur %s\n",message);
    perror("Message d'erreur: ");
  }
  exit(1);
}



int cree_connexion (const char * nom_hote,
		  const char * nom_service,
		  const char * nom_proto,
		  const int num_port)
{
  int sock;
  struct sockaddr_in adresse;
  struct hostent *   hostent  = NULL;
  struct servent *   servent  = NULL;
  struct protoent *  protoent = NULL;
  struct in_addr adresse_ip;
  if (nom_hote != NULL)
    if ((hostent = gethostbyname (nom_hote)) == NULL) 
      if (inet_aton(nom_hote,&adresse_ip))
	{
	if ((hostent = gethostbyaddr((char *) &adresse_ip,
				     sizeof(struct in_addr),
				     AF_INET)) == NULL)
	printf("Rsolution de nom impossible, adresse IP valide\n");
      }
      else  {
	    erreur ("gethostbyname");
	    return (-1);
	  }
  if ((protoent = getprotobyname (nom_proto)) == NULL) {
    erreur ("getprotobyname");
    return (-1);
  }
  if (nom_service != NULL)
    if ((servent = getservbyname (nom_service,
				  protoent -> p_name)) == NULL) {
      erreur ("getservbyname");
      return (-1);
    }
  if ((sock = socket (AF_INET, SOCK_STREAM, 0)) < 0) {
    erreur ("cration douille");
    return (-1);
  }
  memset (& adresse, 0, sizeof (struct sockaddr_in));
  adresse . sin_family = AF_INET;
  if (servent != NULL)
    adresse . sin_port = servent -> s_port;
  else
    if (num_port != 0) adresse.sin_port = htons(num_port);
      else adresse . sin_port = htons (0);
  if (hostent != NULL)
    adresse . sin_addr . s_addr =
      ((struct in_addr *) (hostent -> h_addr)) -> s_addr;
  else adresse . sin_addr . s_addr = adresse_ip.s_addr;
  if (connect (sock, (struct sockaddr *) & adresse,
	    sizeof (struct sockaddr_in)) < 0) {
    close (sock);
    erreur ("localisation douille");
    return (-1);
  }
  return (sock);
}

void ausecours()
{
  printf("clientpartition [-a destination] [-d goal] [-s source]\n");
  printf("                [-p port] [-v] [-h] [destination]\n");
  printf("Ask for \"source\" on \"destination\" and put it in \"goal\".\n");
  printf("source and goal are files or partitions or anything else\n");
  printf("default port: 2003, v if verbose. (Franois Boisson)\n");

  exit(2);
}


main(argc,argv)
int argc;
char **argv;
{
  int douille,i,j;
  char *buffer;
  
  char *hote = NULL;
  char *partition_destination = NULL;
  char *partition_source = NULL;
  int prt = PORT;
  int option;
  unsigned long long int longueur;
  /*  FILE *out_file; */
  int out_file;
 
  buffer = (char *) malloc(TAILLE);
  if (buffer == NULL) erreur("Pas assez de mmoire\n");
  while ((option = getopt(argc,argv,"p:a:d:s:vh")) != -1) {
    switch(option) {
    case 'a':
      hote = optarg;
      break;
    case 'd':
      partition_destination = optarg;
      break;
    case 's':
      partition_source = optarg;
      break;
    case 'p':
      sscanf(optarg,"%d",&prt);
      break;
    case 'v':
      verbeux = 1;
      break;
    case 'h':
    case '?':
    default:
      ausecours(); 
      break;
    }
  }
  if ((hote == NULL) && (optind < argc)) hote = argv[optind];
  if (hote == NULL) ausecours();
  douille = cree_connexion(hote,NULL,"tcp",prt);
/* OK, la connexion est cre, il suffit de lire le flux et
   de l'crire dans la partition destination */
  i = 0;
  read(douille,buffer,1);
  if (buffer[0] != 'C')
    {
      close(douille);
      erreur("Service non conforme\n");
    }
  else if (verbeux) printf("Service trouv\n");
  out_file = open(partition_destination,O_RDWR|O_CREAT|O_TRUNC|O_LARGEFILE,S_IRWXU);
  if (out_file == -1)
    {
      close(douille);
      erreur("Impossible d'ouvrir %s\n",partition_destination);
    }
  write(douille,partition_source,strlen(partition_source)+1);
  longueur = 0;
  while ((j = read(douille,buffer,TAILLE)) > 0)
	 if (write(out_file,buffer,j) < j)
	   {
	     close(out_file);
	     close(douille);
	     printf("%lld octets reus et crits\n",longueur + ((long long int) j));
	     erreur("Erreur  l'criture\n");
	   }
	 else 
	   {
	     longueur += ((long long int) j);
	     if (verbeux) printf ("-> %lld\r",longueur);
	   }
  close(out_file);
  printf("%lld octets reus et crits\n",longueur + ((long long int) j));
  close(douille);
  exit(0);
}


