/* Programme de transfert de fichier/partition minimal
    compiler par diet gcc
   Franois Boisson, juin 2003 */

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <string.h>
#include <stdio.h>
#include <signal.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <unistd.h>
#include <malloc.h>

#define PORT 2003
#define TAILLE (1024*1024)

struct sockaddr_in desc_douille;
int verbeux;

void erreur(message)
char *message;
{
  printf("Erreur %s\n",message);
  perror("Message d'erreur: ");
  exit(1);
}

int cree_douille (const char * nom_hote,
		  const char * nom_service,
		  const char * nom_proto,
		  const int num_port)
{
  int sock;
  struct sockaddr_in adresse;
  struct hostent *   hostent  = NULL;
  struct servent *   servent  = NULL;
  struct protoent *  protoent = NULL;

  if (nom_hote != NULL)
    if ((hostent = gethostbyname (nom_hote)) == NULL) {
      erreur ("gethostbyname");
      return (-1);
    }
  
  if ((protoent = getprotobyname (nom_proto)) == NULL) {
    erreur ("getprotobyname");
    return (-1);
  }
  if (nom_service != NULL)
    if ((servent = getservbyname (nom_service,
				  protoent -> p_name)) == NULL) {
      erreur ("getservbyname");
      return (-1);
    }
  if ((sock = socket (AF_INET, SOCK_STREAM, 0)) < 0) {
    erreur ("cration douille");
    return (-1);
  }
  memset (& adresse, 0, sizeof (struct sockaddr_in));
  adresse . sin_family = AF_INET;
  if (servent != NULL)
    adresse . sin_port = servent -> s_port;
  else
    if (num_port != 0) adresse.sin_port = htons(num_port);
      else adresse . sin_port = htons (0);
  if (hostent != NULL)
    adresse . sin_addr . s_addr =
      ((struct in_addr *) (hostent -> h_addr)) -> s_addr;
  else
    adresse . sin_addr . s_addr = htonl (INADDR_ANY);
  if (bind (sock, (struct sockaddr *) & adresse,
	    sizeof (struct sockaddr_in)) < 0) {
    close (sock);
    erreur ("localisation douille");
    return (-1);
  }
  if (verbeux) printf("C'est parti, douille cree\n");
  return (sock);
}



traite(connexion)
     int connexion;
{
  struct sockaddr_in desc_connexion;
  struct sockaddr_in desc_host;
  int longueur;
  int i,j;
  char *buffer;
  char partition[256];
  int *in_file;

  buffer = (char *) malloc(TAILLE);
  if (buffer == NULL) erreur("Pas assez de mmoire\n");
  if (verbeux) printf("Voila une connexion\n");
  longueur = sizeof(struct sockaddr_in);
  if (getsockname(connexion,
		  (struct sockaddr *) &desc_host,
		  &longueur) < 0)
    erreur("(F): recherche nom serveur");
  if (getpeername(connexion,
		  (struct sockaddr *) &desc_connexion,
		  &longueur) < 0)
    erreur("(F): recherche nom client");
  snprintf(buffer,256,"Connexion de IP : %s(%u)\n",
	 inet_ntoa(desc_connexion.sin_addr),
	 ntohs(desc_connexion.sin_port));
/*  write(connexion,"Salut mec, a roule?\n",21);*/
/* On dit qu'on coute */
  if (verbeux) printf("%s\n",buffer);
  write(connexion,buffer,1);
  partition[0]='r';
  i=0;
  do read(connexion,partition + i,1);
  while (partition[i++]);
  in_file=open(partition,O_RDONLY|O_LARGEFILE);
  if (in_file == -1) {
    printf("Erreur ouverture %s\n",partition);
    return(0);
  } else if (verbeux) printf("%s ouvert\n",partition);
  while ((j=read(in_file,buffer,TAILLE)) > 0)
    write(connexion,buffer,j);
  close(in_file);
}
/* bout enleve
*/


main(argc,argv)
int argc;
char **argv;
{
  int douille,douille_connecte;
  int prt=PORT;
  int longueur;
  struct sockaddr sock_interlocuteur;
  int option;
  while ((option = getopt(argc,argv,"p:a:vh")) != -1) {
    switch(option) {
    case 'p':
      sscanf(optarg,"%d",&prt);
      break;
    case 'v':
      verbeux = 1;
      break;
    case 'h':
    case '?':
    default:
      printf("serveurpartition [-p port] [-v]\n");
      printf("default port: 2003, v if verbose. F.B\n");
      exit(0);
      break;
    }
  }
  douille = cree_douille(NULL,NULL,"tcp",prt);
  if (listen(douille,5) < 0) erreur("coute douille");
  longueur = sizeof(struct sockaddr);
/* vite au processus fils de rester au zombie */
  signal(SIGCHLD,SIG_IGN);

/* attente d'une connexion */


  for(;;) {
    if ((douille_connecte = accept(douille,&sock_interlocuteur,&longueur))< 0)
      erreur("problme  la connexion");


/* Traitement de connexion */
    switch(fork()) {
    case -1 :
      erreur("problme au fork");
    case 0 :
/* le fils */
      close(douille);
/* traitement de la connexion */
      traite(douille_connecte);
/* fin de la connexion */
      close(douille_connecte);
      exit(0);
    default :
      close(douille_connecte);
    }
  }
}


